function [X, featExp] = dof2feat(DOF, joint, paraF, cord, skel)
% Convert the DOF matrix into feature matrix.
%
% Input
%   DOF      -  DOF matrix, (nJ0 x 3) x n
%   joint    -  joint's property
%   paraF
%     feat   -  feature name, {'log'} | 'vel' | 'ang' | 'apos' | 'rpos'
%               'log'   -  logarithm map of quaternion, dim = 3
%               'vel'   -  angluar velocity, dim = 3
%               'mvel'  -  magnitude of angluar velocity, dim = 1
%               'ang'   -  Euler angle, dim = 3
%               'apos'  -  absolute position, dim = 3
%               'rpos'  -  relative position, dim = 3
%     used   -  used joint's name, 1 x nJ cell array
%               {} / [] means keeping all the joints
%   [cord]   -  joint's position
%   [skel]   -  skeleton's structure
%
% Output
%   X        -  expected information, (nJ x dim) x n matrix
%   featExp  -  detailed explanation of extracted feature
%
% History
%   create   -  Feng Zhou, 12-29-2008
%   modify   -  Feng Zhou, 12-29-2008

fprintf('dof -> feat...\n');

% parse parameter
feat = ps(paraF, 'feat', 'log');
used = ps(paraF, 'used', []);

% used joints
jname = joint.name;
if isempty(used)
    used = jname;
elseif ~iscell(used)
    used = {used};
end
nJ = length(used);

ind = zeros(1, nJ); pos = zeros(1, nJ * 3);
for i = 1 : nJ
    ind(i) = strloc(used{i}, jname);
    pos(i * 3 - 2 : i * 3) = ind(i) * 3 - 2 : ind(i) * 3;
end

if strcmp(feat, 'log')
    Q = logq(dof2q(DOF));
    X = Q(pos, :);
    featExp = 'logarithm map of quaternion';

elseif strcmp(feat, 'vel')
    Q = logq(dof2q(DOF));
    n = size(Q, 2);
    X = zeros(nJ * 3, n);
    for i = 1 : n - 1
        X(:, i) = Q(pos, i + 1) - Q(pos, i);
    end
    if n > 1, X(:, end) = X(:, end - 1); end
    featExp = 'angluar velocity';
    
elseif strcmp(feat, 'mvel')
    Q = logq(dof2q(DOF));
    n = size(Q, 2);
    X = zeros(nJ, n);
    for i = 1 : n - 1
        tmp = Q(pos, i + 1) - Q(pos, i);
        for c = 1 : nJ
            X(c, i) = sqrt(sum(tmp((c - 1) * 3 + 1 : c * 3) .^ 2));
        end
    end
    if n > 1, X(:, end) = X(:, end - 1); end
    featExp = 'magnitude of angluar velocity';
    
elseif strcmp(feat, 'ang')
    X = DOF(pos, :);
    featExp = 'Euler angle';
    
elseif strcmp(feat, 'apos')
    ind2 = zeros(1, nJ);
    for i = 1 : nJ
        ind2(i) = strloc(used{i}, skel.tree, 'name');
    end
    
    X = cord(:, ind2, :);
    X = reshape(X, size(X, 1), size(X, 3));
    X = X';
    featExp = 'absolute position';
    
elseif strcmp(feat, 'rpos')
    ind2 = zeros(1, nJ);
    for i = 1 : nJ
        ind2(i) = strloc(used{i}, skel.tree, 'name');
    end
    indRoot = strloc('root', skel.tree, 'name');
    
    X = cord(:, ind2, :);
    X = reshape(X, size(X, 1), size(X, 3));
    X = X';
    
    XRoot = cord(:, indRoot, :);
    XRoot = reshape(XRoot, size(XRoot, 1), size(XRoot, 3));
    XRoot = XRoot';
    
    X = X - repmat(XRoot, nJ, 1);
    
    featExp = 'relative position';
end
