function wsData = mocData(wsSrc, varargin)
% Prepare the motion capture data.
%
% Input
%   wsSrc      -  mocap source
%   varargin
%     save option
%     path     -  new path flag, {'n'} | 'y'
%
% Output
%   wsData
%     DOF      -  Degree of Freedom, dim x nF0
%     Q        -  quaternion (continuous) feature, dim x nF
%     X        -  1-D (discrete) feature, 1 x nF
%     DX       -  1-D (discrete) feature without temporal reduction, 1 x nF0
%     stR
%
% History
%   create     -  Feng Zhou, 12-29-2008
%   modify     -  Feng Zhou, 12-31-2008

% parameter
[path, isSave] = psSave(varargin, 'subfix', 'data', 'folder', 'mocap/mat');

% load
if ~isSave && exist(path, 'file')
    tmpMat = loadMat(path);
    wsData = tmpMat.wsData;
    return;
end
fprintf('acquiring the mocap data...\n');

% sources
sources = wsSrc.sources;
n = length(sources);
paraF = wsSrc.paraF;

% DOFs
DOFs = cell(1, n);
for i = 1 : n
    wsDOF = mocDOF(sources{i}, paraF);
    DOFs{i} = wsDOF.DOF;
    joint = wsDOF.joint;
end

% concatenate DOF if necessary
nFs = cellDim(DOFs, 2);
st = n2st(nFs);
DOF = dofJoin(DOFs);

% change path
isPath = psY(varargin, 'path', 'n');
if isPath
    [DOF, angV, angP] = dofPathNew(DOF, st);
end
wsData.DOF = DOF; wsData.joint = joint;

% parse skel
isSkel = psY(varargin, 'skel', 'y');
if isSkel
    skel0 = asf2skel(sources{1});
    [skel, channels] = dof2skel(DOF, joint, skel0);
    if isPath
        [cord, cone] = parseCoordinate(skel, channels, angV, angP);
    else
        [cord, cone] = parseCoordinate(skel, channels);
    end

    wsData.skel = skel;
    wsData.cord = cord;
    wsData.cone = cone;
end

% feature
[DOF2, joint2] = dofFilter(DOF, joint, paraF);
Q = dof2feat(DOF2, joint2, paraF);

% temporal reduction
[X, stR, DX] = tempoReduce(Q, wsSrc.para);
Q = stPick(Q, stR);
wsData.Q = Q; wsData.DX = DX; wsData.X = X; wsData.stR = stR;

% save
save(path, 'wsData');